unit MSWord;
interface
uses
  Windows, Classes, ActiveX, Word97, Log;
type
  TWordEventSink = class(TInterfacedObject, IUnknown, IDispatch)
  private
    FOwner : TObject;
    FAppDispatch: IDispatch;
    FDocDispatch: IDispatch;
    FAppDispIntfIID: TGUID;
    FDocDispIntfIID: TGUID;
    FAppConnection: Integer;
    FDocConnection: Integer;
    FOnQuit : TNotifyEvent;
    FOnDocumentChange : TNotifyEvent;
    FOnNewDocument : TNotifyEvent;
    FOnOpenDocument : TNotifyEvent;
    FOnCloseDocument : TNotifyEvent;
  protected
    { IUnknown }
    function QueryInterface(const IID: TGUID; out Obj): HRESULT; stdcall;
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
    { IDispatch }
    function GetTypeInfoCount(out Count: Integer): HRESULT; stdcall;
    function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HRESULT; stdcall;
    function GetIDsOfNames(const IID: TGUID; Names: Pointer;
      NameCount, LocaleID: Integer; DispIDs: Pointer): HRESULT; stdcall;
    function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
      Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HRESULT; stdcall;
  public
    constructor Create(AnOwner: TObject; AnAppDispatch: IDispatch; const AnAppDispIntfIID, ADocDispIntfIID: TGUID);
    destructor Destroy; override;
    property OnQuit : TNotifyEvent read FOnQuit write FOnQuit;
    property OnDocumentChange : TNotifyEvent read FOnDocumentChange write FOnDocumentChange;
    property OnNewDocument : TNotifyEvent read FOnNewDocument write FOnNewDocument;
    property OnOpenDocument : TNotifyEvent read FOnOpenDocument write FOnOpenDocument;
    property OnCloseDocument : TNotifyEvent read FOnCloseDocument write FOnCloseDocument;
  end;


  TMSWordObject = class
   private
    FWordApp : _Application;     // in Word97.pas
    FEventSink : TWordEventSink;
    function GetCaption : String;
    function GetVisible : Boolean;
    procedure SetVisible(Value : Boolean);
    function GetOnQuit : TNotifyEvent;
    procedure SetOnQuit(Value : TNotifyEvent);
    function GetOnDocumentChange : TNotifyEvent;
    procedure SetOnDocumentChange(Value : TNotifyEvent);
    function GetOnNewDocument: TNotifyEvent;
    procedure SetOnNewDocument(Value : TNotifyEvent);
    function GetOnOpenDocument: TNotifyEvent;
    procedure SetOnOpenDocument(Value : TNotifyEvent);
    function GetOnCloseDocument: TNotifyEvent;
    procedure SetOnCloseDocument(Value : TNotifyEvent);
  public
    constructor Create;
    destructor Destroy; override;
    procedure NewDoc(Template : String);
    procedure CloseDoc;
    procedure InsertText(Text : String);
    procedure Print;
    procedure SaveAs(Filename : String; wdFormat : dword );
    procedure SetCaption(Value : String);
    procedure SaveAsRTF(Filename : String);
    procedure OpenFile(FileName : String );
    Function  CheckSpelling( Text : String ):dword;
  published
    property WordApp : _Application read FWordApp;
    property Caption : String read GetCaption write SetCaption;
    property Visible : Boolean read GetVisible write SetVisible;
    property OnQuit : TNotifyEvent read GetOnQuit write SetOnQuit;
    property OnDocumentChange : TNotifyEvent read GetOnDocumentChange write SetOnDocumentChange;
    property OnNewDocument : TNotifyEvent read GetOnNewDocument write SetOnNewDocument;
    property OnOpenDocument : TNotifyEvent read GetOnOpenDocument write SetOnOpenDocument;
    property OnCloseDocument : TNotifyEvent read GetOnCloseDocument write SetOnCloseDocument;
  end;

implementation

uses
  ComObj;

{ TWordEventSink implementation }

constructor TWordEventSink.Create(AnOwner : TObject; AnAppDispatch: IDispatch; const AnAppDispIntfIID, ADocDispIntfIID: TGUID);
begin
  try
  inherited Create;

  FOwner := AnOwner;
  FAppDispIntfIID := AnAppDispIntfIID;
  FDocDispIntfIID := ADocDispIntfIID;
  FAppDispatch := AnAppDispatch;

  // Hook the sink up to the automation server (Word97)
  InterfaceConnect(FAppDispatch,FAppDispIntfIID,Self,FAppConnection);
  except
   AddToLog('WORD','Except creat MS Word COM Interface');
  end; 
end;

destructor TWordEventSink.Destroy;
begin
  try
  // Unhook the sink from the automation server (Word97)
  InterfaceDisconnect(FAppDispatch,FAppDispIntfIID,FAppConnection);

  inherited Destroy;
  except
   AddToLog('WORD','Except destroy MS Word COM Interface');
  end;
end;

function TWordEventSink.QueryInterface(const IID: TGUID; out Obj): HRESULT;
begin
 try
  // We need to return the two event interfaces when they're asked for
  Result := E_NOINTERFACE;
  if GetInterface(IID,Obj) then
    Result := S_OK;
  if IsEqualGUID(IID,FAppDispIntfIID) and GetInterface(IDispatch,Obj) then
    Result := S_OK;
  if IsEqualGUID(IID,FDocDispIntfIID) and GetInterface(IDispatch,Obj) then
    Result := S_OK;
  except
   AddToLog('WORD','Except query MS Word COM Interface');
  end;
end;

function TWordEventSink._AddRef: Integer;
begin
 try
// Skeleton implementation
  Result := 2;
  except
   AddToLog('WORD','Except');
  end;
end;

function TWordEventSink._Release: Integer;
begin
 try
// Skeleton implementation
  Result := 1;
  except
   AddToLog('WORD','Except');
  end;
end;

function TWordEventSink.GetTypeInfoCount(out Count: Integer): HRESULT;
begin
 try
// Skeleton implementation
  Count  := 0;
  Result := S_OK;
  except
   AddToLog('WORD','Except');
  end;
end;

function TWordEventSink.GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HRESULT;
begin
 try
// Skeleton implementation
  Result := E_NOTIMPL;
  except
   AddToLog('WORD','Except');
  end;
end;

function TWordEventSink.GetIDsOfNames(const IID: TGUID; Names: Pointer;
  NameCount, LocaleID: Integer; DispIDs: Pointer): HRESULT;
begin
 try
// Skeleton implementation
  Result := E_NOTIMPL;
  except
   AddToLog('WORD','Except');
  end;
end;

function TWordEventSink.Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer;
  Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HRESULT;
begin
 try
  // Fire the different event handlers when
  // the different event methods are invoked
  case DispID of
    2 : if Assigned(FOnQuit) then
          FOnQuit(FOwner);
    3 : begin
          if Assigned(FOnDocumentChange) then
            FOnDocumentChange(FOwner);
          // When we see a document change, we also need to disconnect the
          // sink from the old document, and hook it up to the new document
          InterfaceDisconnect(FDocDispatch,FDocDispIntfIID,FDocConnection);
          try
            FDocDispatch := _Application(FAppDispatch).ActiveDocument;
            InterfaceConnect(FDocDispatch,FDocDispIntfIID,Self,FDocConnection);
          except;
         end;
        end;
    4 : if Assigned(FOnNewDocument) then
          FOnNewDocument(FOwner);
    5 : if Assigned(FOnOpenDocument) then
          FOnOpenDocument(FOwner);
    6 : if Assigned(FOnCloseDocument) then
          FOnCloseDocument(FOwner);
  end;

  Result := S_OK;
  except
   AddToLog('WORD','Except invoke');
  end;
end;

//////////////////////////////////////////////////
constructor TMSWordObject.Create;
begin
 try
  FWordApp := CoWordApplication.Create;
  except
   AddToLog('WORD','Except object create');
  end;
end;

destructor TMSWordObject.Destroy;
var
  SaveChanges,
  OriginalFormat,
  RouteDocument  : OleVariant;
begin
 try
  SaveChanges := WdDoNotSaveChanges;
  OriginalFormat := 0;//UnAssigned;
  RouteDocument := 0;//UnAssigned;
  try
    FWordApp.Quit(SaveChanges,OriginalFormat,RouteDocument);
  except
   AddToLog('WORD','Except MS Word quit');
  end;
  FEventSink := nil;
  inherited Destroy;
  except
   AddToLog('WORD','Except object destroy');
  end;
end;

function TMSWordObject.GetVisible : Boolean;
begin
 try
  Result := FWordApp.Visible;
  except
   AddToLog('WORD','Except visible');
  end;
end;

procedure TMSWordObject.SetCaption(Value : String);
begin
 try
  FWordApp.Caption := Value;
  except
   AddToLog('WORD','Except set caption');
  end;
end;

function TMSWordObject.GetCaption : String;
begin
 try
  Result := FWordApp.Caption;
  except
   AddToLog('WORD','Except get caption');
  end;
end;

procedure TMSWordObject.SetVisible(Value : Boolean);
begin
 try
  FWordApp.Visible := Value;
  except
   AddToLog('WORD','Except set visible');
  end;
end;

function TMSWordObject.GetOnQuit : TNotifyEvent;
begin
 try
  Result := FEventSink.OnQuit;
  except
   AddToLog('WORD','Except get quit');
  end;
end;

procedure TMSWordObject.SetOnQuit(Value : TNotifyEvent);
begin
 try
  FEventSink.OnQuit := Value;
  except
   AddToLog('WORD','Except set quit event');
  end;
end;

function TMSWordObject.GetOnDocumentChange : TNotifyEvent;
begin
 try
  Result := FEventSink.OnDocumentChange;
  except
   AddToLog('WORD','Except set document event');
  end;
end;

procedure TMSWordObject.SetOnDocumentChange(Value : TNotifyEvent);
begin
 try
  FEventSink.OnDocumentChange := Value;
  except
   AddToLog('WORD','Except document cheng event');
  end;
end;

function TMSWordObject.GetOnNewDocument : TNotifyEvent;
begin
 try
  Result := FEventSink.OnNewDocument;
  except
   AddToLog('WORD','Except event');
  end;
end;

procedure TMSWordObject.SetOnNewDocument(Value : TNotifyEvent);
begin
 try
  FEventSink.OnNewDocument := Value;
  except
   AddToLog('WORD','Except event');
  end;
end;

function TMSWordObject.GetOnOpenDocument : TNotifyEvent;
begin
 try
  Result := FEventSink.OnOpenDocument;
  except
   AddToLog('WORD','Except event');
  end;
end;

procedure TMSWordObject.SetOnOpenDocument(Value : TNotifyEvent);
begin
 try
  FEventSink.OnOpenDocument := Value;
  except
   AddToLog('WORD','Except event');
  end;
end;

function TMSWordObject.GetOnCloseDocument : TNotifyEvent;
begin
 try
  Result := FEventSink.OnCloseDocument;
  except
   AddToLog('WORD','Except event');
  end;
end;

procedure TMSWordObject.SetOnCloseDocument(Value : TNotifyEvent);
begin
 try
  FEventSink.OnCloseDocument := Value;
  except
   AddToLog('WORD','Except event');
  end;
end;

procedure TMSWordObject.InsertText(Text : String);
begin
 try
  FWordApp.Selection.TypeText(Text);
  except
   AddToLog('WORD','Except event');
  end;
end;

procedure TMSWordObject.NewDoc(Template : String);
var
  DocTemplate,
  NewTemplate : OleVariant;
begin
 try
  DocTemplate := Template;
  NewTemplate := False;
  FWordApp.Documents.Add(DocTemplate,NewTemplate);
  except
   AddToLog('WORD','Except new document create');
  end;
end;

////////// PUBLIC /////////////////////////////////////
procedure TMSWordObject.CloseDoc;
var
  SaveChanges,
  OriginalFormat,
  RouteDocument  : OleVariant;
begin
 try
  SaveChanges := WdDoNotSaveChanges;
  OriginalFormat := 0;//UnAssigned;
  RouteDocument := 0;//UnAssigned;
  FWordApp.ActiveDocument.Close(SaveChanges,OriginalFormat,RouteDocument);
  except
   AddToLog('WORD','Except close document');
  end;
end;

procedure TMSWordObject.Print;
begin
 try
  OleVariant(FWordApp).PrintOut;
  except
   AddToLog('WORD','Except document print');
  end;
end;

procedure TMSWordObject.SaveAs(Filename : String; wdFormat : dword );
begin
 try
  OleVariant(FWordApp).ActiveDocument.SaveAs( FileName ,wdFormat );
  except
   AddToLog('WORD','Except document save');
  end;
end;

procedure TMSWordObject.OpenFile(FileName : String );
begin
 try
    OleVariant(FWordApp).Documents.Open(FileName);
  except
   AddToLog('WORD','Except open document');
  end;
end;

procedure TMSWordObject.SaveAsRTF(FileName : String);
begin
 try
  OleVariant(FWordApp).ActiveDocument.SaveAs( FileName ,wdFormatRTF);
  except
   AddToLog('WORD','Except save as rtf');
  end;
end;

Function TMSWordObject.CheckSpelling( Text : String ):dword;
begin
 try
  result := OleVariant(FWordApp).CheckSpelling( WideString(Text ));
  except
   AddToLog('WORD','Except check spelling');
  end;
end;

end.

